#!/usr/bin/env python
# -*- coding: utf-8; py-indent-offset:4 -*-

from __future__ import (absolute_import, division, print_function, unicode_literals)

import os


directory = 'test string'
str_pt1 = f'from .'
str_pt2 = f' import *'

if directory is None:
    directory = os.getcwd()


# Does directory exist
def is_directory(path):
    if os.path.isdir(path):
        return True
    else:
        return False

# Does file exist
def is_file(path):
    if os.path.isfile(path):
        return True
    else:
        return False


for pyfile in os.listdir(directory):
    # only look at python files that are not __init__ files
    if 'py' in pyfile and '__init__' not in pyfile:


# def create_init(directory=None):






    try:
        pass

    except FileNotFoundError as file_not_found:
        return f''




def loop_through_directory(directory, fn):


    for pyfile in os.listdir(directory)



            package = pyfile[:pyfile.find('.')]
            pyfile_list.append()

            print(''.join([str_pt1, package, str_pt2]))


def relative_import_str(*args):
    return ''.join(arg for arg in args)



#!/usr/bin/env python
# -*- coding: utf-8; py-indent-offset:4 -*-

from . import utils as utils

from .utils import num2date, date2num, time2num, num2time

from .store import Store
from . import broker as broker
from .timer import *
from . import utils as utils

from . import feeds as feeds
from . import indicators as indicators



# Load contributed indicators and studies
import backtrader.indicators.contrib
import backtrader.studies.contrib



#!/usr/bin/env python
# -*- coding: utf-8; py-indent-offset:4 -*-

from __future__ import (absolute_import, division, print_function,
                        unicode_literals)

import sys

from .area_between import area_between
from .crossover import crossover
from .draw_candlesticks import draw_candlesticks
from .fill_values import fill_values
from .group_candles import group_candles
from .intersection import intersection



#!/usr/bin/env python
# -*- coding: utf-8; py-indent-offset:4 -*-

import pandas as pd
import numpy as np


def group_candles(df, interval):
    ''' Combine candles so instead of needing one dataset for each time interval,
        you can form time intervals using more precise data.

    Example: You have 15-min candlestick data but want to test a strategy based
        on 1-hour candlestick data  (interval=4).
    '''

    columns = ['date', 'open', 'high', 'low', 'close', 'volume']
    candles = df[columns].values
    results = []
    for i in range(0, len(df)-interval, interval):
        results.append([
            candles[i, 0],                      # date
            candles[i, 1],                      # open
            candles[i:i+interval, 2].max(),     # high
            candles[i:i+interval, 3].min(),     # low
            candles[i+interval, 4],             # close
            candles[i:i+interval, 5].sum()      # volume
        ])
    return pd.DataFrame(results, columns=columns)





@pd_series_to_np_array
def roc(src, n=14):
    ''' Returns the rate of change in price over n periods '''

    _roc = np.zeros(src.shape)
    _roc[n:] = (np.diff(src, n) ) / src[:-n]

    return _roc



#!/usr/bin/env python
# -*- coding: utf-8; py-indent-offset:4 -*-
from __future__ import (absolute_import, division, print_function, unicode_literals)

from .renko import *






# ------------------------------------------------------------------------------


def plot():

    fig, ax = plt.subplots(figsize=(30,10))

    x = self.df['date']
    plt.plot(x, self.ichimoku['tenkan'], color='blue')
    plt.plot(x, self.ichimoku['kijun'], color='maroon')
    plt.plot(x, self.ichimoku['senkou_a'], color='green', linewidth=0.5)
    plt.plot(x, self.ichimoku['senkou_b'], color='red', linewidth=0.5)

    draw_candlesticks(ax, self.df)

    fig.suptitle('Ichimoku', fontsize=30)
    plt.ylabel('BTC price ($)')
    plt.rc('axes', labelsize=20)
    plt.rc('font', size=18)

    ax.xaxis.set_major_locator(mdates.MonthLocator())
    ax.xaxis.set_major_formatter(mdates.DateFormatter('%m/%d/%y'))
    plt.xticks(rotation=45)

    plt.fill_between(
        x, self.ichimoku['senkou_a'], self.ichimoku['senkou_b'],
        where=self.ichimoku['senkou_a'] >= self.ichimoku['senkou_b'],
        facecolor='limegreen',
        interpolate=True
    )
    plt.fill_between(
        x, self.ichimoku['senkou_a'], self.ichimoku['senkou_b'],
        where=self.ichimoku['senkou_a'] <= self.ichimoku['senkou_b'],
        facecolor='salmon',
        interpolate=True
    )
    return plt.show()





@pd_series_to_np_array
def sma(src, n=14):
    ''' Returns the "simple moving average" for a list across n periods'''

    summed = rolling(src, fn='sum', n=n)
    _sma = summed / n

    return _sma



#!/usr/bin/env python
# -*- coding: utf-8; py-indent-offset:4 -*-

from datetime import datetime, timedelta
import matplotlib.dates as mdates
pd.plotting.register_matplotlib_converters()

# TODO: fix imports

from .utils import *
from .ta import *

class Ichimoku:
    def __init__(self, df, period=None):
        self.df = df.copy()
        if period:
            self.df = group_candles(self.df, period)
        self.df['date'] = self.df['date'].apply(lambda x: datetime.strptime(x, '%Y-%m-%d %H:%M:%S'))
        self.ichimoku = {}


    def build(self, tenkan_period, kijun_period, senkou_b_period, displacement):
        ''' Create Ichimoku data '''

        self._validate_data(senkou_b_period)
        self._extend_window(displacement)

        # Tenkan (conversion line) = (highest high + highest low)/2 for the past x periods
        # Kijun (base line) = (highest high + lowest low)/2 for the past x*3 periods
        self._build_lines(tenkan=tenkan_period, kijun=kijun_period)

        # Chikou (lagging span) = Current closing price time-shifted backwards x*3 periods
        # Senkou span A (leading span A) = (tenkan + kijun)/2 time-shifted forwards x*3 periods
        # Senkou span B (leading span B) = (highest high + lowest low)/2 for past x*6 periods, shifted forwards x*3 periods
        self._build_spans(senkou_b_period=senkou_b_period, displacement=displacement)
        return


    def _validate_data(self, period):
        ''' Make sure we have enough data to form the cloud '''
        if period > len(self.df):
            raise AssertionError('Unable to form cloud.  Make sure the dataset has more than {} records.'.format(period))


    def _extend_window(self, displacement):
        last_date = self.df['date'].iat[-1]
        window = last_date - self.df['date'].iat[-2]
        new_dates = [last_date + window*(1+i) for i in range(displacement)]
        self.df = self.df.append(pd.DataFrame(new_dates, columns=['date']), ignore_index=True, sort=False)
        return


    def _build_lines(self, **kwargs):
        for key, val in kwargs.items():
            high = sma(self.df['high'], val, 'max')
            low = sma(self.df['low'], val, 'min')
            self.ichimoku[key] = (high + low)/2
        return


    def _build_spans(self, senkou_b_period, displacement):
        self.ichimoku['chikou'] = self.df['close'].shift(-displacement)
        self.ichimoku['senkou_a'] = (self.ichimoku['tenkan'] + self.ichimoku['kijun']) / 2

        self._build_lines(senkou_b=senkou_b_period)

        self.ichimoku['senkou_a'] = self.ichimoku['senkou_a'].shift(displacement)
        self.ichimoku['senkou_b'] = self.ichimoku['senkou_b'].shift(displacement)

        for indicator in ['tenkan', 'kijun', 'chikou', 'senkou_a', 'senkou_b']:
            self.ichimoku[indicator] = self.ichimoku[indicator][senkou_b_period+displacement:]

        self.df = self.df[senkou_b_period+displacement:]
        return


    def plot(self):
        fig, ax = plt.subplots(figsize=(30,10))

        x = self.df['date']
        plt.plot(x, self.ichimoku['tenkan'], color='blue')
        plt.plot(x, self.ichimoku['kijun'], color='maroon')
        plt.plot(x, self.ichimoku['senkou_a'], color='green', linewidth=0.5)
        plt.plot(x, self.ichimoku['senkou_b'], color='red', linewidth=0.5)

        draw_candlesticks(ax, self.df)

        fig.suptitle('Ichimoku', fontsize=30)
        plt.ylabel('BTC price ($)')
        plt.rc('axes', labelsize=20)
        plt.rc('font', size=18)

        ax.xaxis.set_major_locator(mdates.MonthLocator())
        ax.xaxis.set_major_formatter(mdates.DateFormatter('%m/%d/%y'))
        plt.xticks(rotation=45)

        plt.fill_between(
            x, self.ichimoku['senkou_a'], self.ichimoku['senkou_b'],
            where=self.ichimoku['senkou_a'] >= self.ichimoku['senkou_b'],
            facecolor='limegreen',
            interpolate=True
        )
        plt.fill_between(
            x, self.ichimoku['senkou_a'], self.ichimoku['senkou_b'],
            where=self.ichimoku['senkou_a'] <= self.ichimoku['senkou_b'],
            facecolor='salmon',
            interpolate=True
        )
        return plt.show()



#!/usr/bin/env python
# -*- coding: utf-8; py-indent-offset:4 -*-

import numpy as np
from .wrappers import pd_series_to_np_array


@pd_series_to_np_array
def area_between(line1, line2):
    ''' Return the area between line1 and line2 '''

    diff = line1 - line2
    x1 = diff[:-1]
    x2 = diff[1:]

    triangle_area = abs(x2 - x1) * .5
    square_area = np.amin(zip(x1, x2), axis=1)

    return np.sum([triangle_area, square_area])



#!/usr/bin/env python
# -*- coding: utf-8; py-indent-offset:4 -*-

from .utils import *
from .ta import *

class Renko:
    def __init__(self, df):
        try:
            self.date = pd.to_datetime(df['date'])
        except:
            self.date = df['date']

        self.date = self.date.shift(1).values

        self.high = df['high'].values
        self.low = df['low'].values
        self.close = df['close'].values


    def set_brick_size(self, brick_size=None, auto=True, atr_period=14):
        ''' Setting brick size '''

        if len(self.close) < atr_period:
            raise ValueError('ATR period is longer than historical data.')

        self.brick_size = self._optimize_brick_size(auto, brick_size, atr_period)
        return self.brick_size


    def _optimize_brick_size(self, auto, brick_size, atr_period):
        ''' Helper function to get optimal brick size based on ATR '''

        if auto and not brick_size:
            average_true_range = atr(self.high, self.low, self.close, atr_period)
            brick_size = np.median(average_true_range)

        return brick_size


    def build(self):
        ''' Create Renko data '''

        units = self.close[0] // self.brick_size
        start_price = units * self.brick_size

        # Create Genesis block
        self.renko = {
            'index': [0],
            'date': [self.date[1]],
            'price': [start_price],
            'direction': [0]
        }

        # Run price feed through our renko model
        for i in range(1, len(self.close)):
            self._apply_renko(i)

        return


    def _apply_renko(self, i):
        ''' Determine if there are any new bricks to paint with current price '''

        num_bricks = 0

        gap = (self.close[i] - self.renko['price'][-1]) // self.brick_size
        direction = np.sign(gap)
        # No gap means there's not a new brick
        if direction == 0:
            return
        # Add brick(s) in the same direction
        if (gap > 0 and self.renko['direction'][-1] >= 0) \
        or (gap < 0 and self.renko['direction'][-1] <= 0):
            num_bricks = gap
        # Gap >= 2 or -2 and opposite renko direction means we're switching brick direction
        elif np.abs(gap) >= 2:
            num_bricks = gap - 2*direction
            self._update_renko(i, direction, 2)

        for brick in range(abs(int(num_bricks))):
            self._update_renko(i, direction)

        return


    def _update_renko(self, i, direction, brick_multiplier=1):
        ''' Append price and new block to renko dict '''

        renko_price = self.renko['price'][-1] + (direction * brick_multiplier * self.brick_size)
        self.renko['index'].append(i)
        self.renko['price'].append(renko_price)
        self.renko['direction'].append(direction)
        self.renko['date'].append(self.date[i])
        return



#!/usr/bin/env python
# -*- coding: utf-8; py-indent-offset:4 -*-

# TODO: imports

@pd_series_to_np_array
def murrey_math_oscillator(src, n=2):
    ''' Returns the Murrey Math Oscillator of the close '''

    # Donchian channel
    highest = rolling(src, fn='max', n=n)
    lowest = rolling(src, fn='min', n=n)

    rng = highest - lowest

    # Oscillator
    rng_multiplier = rng * .125
    midline = lowest + rng_multiplier * 4

    _murrey_math_oscillator = (src - midline) / rng
    _murrey_math_oscillator[:n] = 0

    return _murrey_math_oscillator



#!/usr/bin/env python
# -*- coding: utf-8; py-indent-offset:4 -*-

import matplotlib.dates as mdates
pd.plotting.register_matplotlib_converters()

# TODO: imports

class Bollinger:
    def __init__(self, close, date=None, period=None):
        self.close = close
        self.date = date
        if period:
            self.close = group_candles(self.close, period)
        # self.df['date'] = self.df['date'].apply(lambda x: datetime.strptime(x, '%Y-%m-%d %H:%M:%S'))
        self.bollinger = {}


    def build(self, n=20, ndev=2):
        self.n = n
        self.ndev = ndev

        self._validate_data()
        self._apply_bollinger()


    def _validate_data(self):
        ''' Make sure we have more rows of data than the rolling average size '''
        if self.n > len(self.close):
            raise AssertionError('Moving average cannot be larger than number of records.')


    def _apply_bollinger(self):
        self.bollinger['sma'] = sma(self.close, window=self.n)
        rng = self.ndev * sdev(self.close, window=self.n)

        self.bollinger['h_band'] = self.bollinger['sma'] + rng
        self.bollinger['l_band'] = self.bollinger['sma'] - rng


    def plot(self):
        fig, ax = plt.subplots(1, figsize=(20, 10))

        if date:
            x = self.date
        else:
            x = range(len(self.close))
        plt.plot(x, self.bollinger['sma'], label='Simple Moving Average', color='orange')
        plt.plot(x, self.bollinger['l_band'], color='blue', label=None)
        plt.plot(x, self.bollinger['h_band'], color='blue')

        draw_candlesticks(ax, self.df)

        plt.rc('axes', labelsize=20)
        plt.rc('font', size=18)

        ax.xaxis.set_major_formatter(mdates.DateFormatter('%m/%d/%y'))
        ax.xaxis.set_major_locator(mdates.MonthLocator())
        plt.xticks(rotation=45)

        fig.suptitle('Bollinger Bands', fontsize=30)
        plt.ylabel('BTC price ($)')
        plt.legend(['{}MA'.format(self.n)])

        return plt.show()



#!/usr/bin/env python
# -*- coding: utf-8; py-indent-offset:4 -*-

# TODO: imports

def double_smooth(src, slow=25, fast=13):
    ''' Returns the smoothed value of two EMAs '''

    first_smooth = ema(src, n=slow)
    _double_smooth = ema(first_smooth, n=fast)

    return _double_smooth.values
